// Configuration for your app
const path = require('path')
const ESLintPlugin = require('eslint-webpack-plugin')
const { merge } = require('webpack-merge')
const transformAssetUrls = require('quasar/dist/transforms/loader-asset-urls.json')

module.exports = function (ctx) {
  return {
    // app boot file (/src/boot)
    // --> boot files are part of "main.js"
    boot: [
      'components',
      'custom-boot'
    ],

    css: [
      'app.sass'
    ],

    extras: [
      'material-icons'
    ],

    // preFetch: true,

    build: {
      vueRouterMode: 'history',
      showProgress: ctx.dev,
      distDir: 'dist/quasar.dev',
      // analyze: true,
      extendWebpack (cfg) {
        const nodeModulesPath = '../../../../..'
        cfg.resolve.alias = {
          ...cfg.resolve.alias, // This adds the existing alias

          // 替换原有的地址，使用现有的添加-D后缀  ----  boot src app 目录不可以修改
          'src-D': path.resolve(__dirname, './src'),
          'app-D': path.resolve(__dirname, '/'),
          'components-D': path.resolve(__dirname, './src/components'),
          'layouts-D': path.resolve(__dirname, './src/layouts'),
          'pages-D': path.resolve(__dirname, './src/pages'),
          'assets-D': path.resolve(__dirname, './src/assets'),
          'boot-D': path.resolve(__dirname, './src/boot'),
          'resource-D': path.resolve(__dirname, nodeModulesPath+'/docs/resource'),
          '@': path.resolve(__dirname, '../src'),
          components: path.resolve(__dirname, nodeModulesPath+'/src/components'),
          layouts: path.resolve(__dirname, nodeModulesPath+'/src/layouts'),
          pages: path.resolve(__dirname, nodeModulesPath+'/src/pages'),
          assets: path.resolve(__dirname, nodeModulesPath+'/src/assets')
        }
      },

      chainWebpack (chain, { isServer }) {
        chain.plugin('eslint-webpack-plugin')
          .use(ESLintPlugin, [{
            extensions: [ 'js', 'vue' ],
            exclude: [ 'node_modules', '.md.js' ]
          }])

        chain.resolve.alias
          .merge({
            examples: path.resolve(__dirname, 'src/examples'),
            markup: path.resolve(__dirname, 'src/markup')
          })

        chain.module.rule('pug')
          .test(/\.pug$/)
          .use('pug-loader').loader('pug-plain-loader')

        const rule = chain.module.rule('md')
          .test(/\.md$/)

        rule.use('v-loader')
          .loader('vue-loader')
          .options({
            ...(
              isServer === true
                ? { ssr: true, directiveTransforms: require('quasar/dist/ssr-directives/index.js') }
                : {}
            ),
            transformAssetUrls: merge({
              base: null,
              includeAbsolute: false,
              tags: {
                video: [ 'src', 'poster' ],
                source: ['src'],
                img: ['src'],
                image: [ 'xlink:href', 'href' ],
                use: [ 'xlink:href', 'href' ]
              }
            }, transformAssetUrls)
          })

        rule.use('md-loader')
          .loader(require.resolve('./build/md-loader'))
      }
    },

    devServer: {
      // https: true,
      port: 9090,
      open: true // opens browser window automatically
    },

    framework: {
      iconSet: 'svg-mdi-v6',

      config: {
        loadingBar: {
          color: 'brand-primary',
          size: '4px'
        }
      },

      components: [
        'QMarkupTable', // required md-plugin-table
        'QBtn', // used directly in some .md files
        'QBadge', // used directly in some .md files
        'QSeparator' // used directly in some .md files
      ],

      plugins: [
        'AddressbarColor',
        'AppFullscreen',
        'AppVisibility',
        'BottomSheet',
        'Cookies',
        'Dark',
        'Dialog',
        'Loading',
        'LoadingBar',
        'LocalStorage',
        'Meta',
        'Notify',
        'Platform',
        'Screen',
        'SessionStorage'
      ]
    },

    animations: [ 'fadeIn', 'fadeOut' ],

    vendor: {
      remove: [
        'quasar/dist/api'
      ]
    }
  }
}
