import DocLayout from 'layouts-D/DocLayout.vue'
import getListingComponent from 'components-D/getListingComponent.js'
import menu from 'resource-D/route/menu.js'
import extendRoutes from 'resource-D/extend/routes.js'

const docsPages = []

function parseMenuNode (node, __path) {
  const prefix = __path + (node.path !== void 0 ? '/' + node.path : '')

  if (node.children !== void 0) {
    prefix !== '/start' && docsPages.push({
      path: prefix,
      component: getListingComponent(
        node.name,
        node.children.map(node => {
          const to = node.external === true
            ? node.path
            : (
                prefix + (
                  node.path !== void 0
                    ? '/' + node.path
                    : (node.listPath !== void 0 ? '/' + node.listPath : '')
                )
              )

          if (node.external !== true && node.listPath !== void 0) {
            docsPages.push({
              path: to,
              component: getListingComponent(
                node.name,
                node.children.map(node => ({
                  title: node.name,
                  to: prefix + (node.path !== void 0 ? '/' + node.path : ''),
                  page: true
                }))
              )
            })
          }

          return {
            title: node.name,
            to,
            page: node.children === void 0
          }
        })
      )
    })

    node.children.forEach(node => parseMenuNode(node, prefix))
  }
  else if (node.external !== true) {
    docsPages.push({
      path: prefix,
      component: () => import('resource-D/md/' + prefix.substring(1) + '.md')
    })
  }
}

menu.forEach(node => {
  parseMenuNode(node, '')
})

const routes = [
  {
    path: '/',
    component: () => import('pages-D/Landing.vue')
  },
  {
    path: '/start',
    redirect: '/start/how-to-use-vue'
  },
  {
    path: '/',
    component: DocLayout,
    children: docsPages
  },

  // externals
  {
    path: '/layout-builder',
    component: () => import('layouts-D/LayoutBuilder.vue')
  },
  ...extendRoutes,
  // Always leave this as last one
  {
    path: '/:catchAll(.*)*',
    component: DocLayout,
    children: [{
      path: '',
      component: () => import('pages-D/Error404.vue')
    }]
  }
]

export default routes
