/*
 * @Author: qs
 * @Date: 2022-03-07 16:57:09
 * @LastEditTime: 2022-03-08 13:25:41
 * @Description: 默认的拦截处理
 */
const toString = Object.prototype.toString
const hasOwn = Object.prototype.hasOwnProperty
const class2type = {}

'Boolean Number String Function Array Date RegExp Object'.split(' ').forEach(name => {
  class2type['[object ' + name + ']'] = name.toLowerCase()
})

function type (obj) {
  return obj === null ? String(obj) : class2type[toString.call(obj)] || 'object'
}

function isPlainObject (obj) {
  if (!obj || type(obj) !== 'object') {
    return false
  }

  if (obj.constructor &&
    !hasOwn.call(obj, 'constructor') &&
    !hasOwn.call(obj.constructor.prototype, 'isPrototypeOf')) {
    return false
  }

  let key
  for (key in obj) {}

  return key === undefined || hasOwn.call(obj, key)
}
// 取自quasar的extend，用于对象的深度复制，但是注意方法
const extend = function (d,t,c) {
  let options
  let name
  let src
  let copy
  let copyIsArray
  let clone
  let target = arguments[0] || {}
  let i = 1
  let deep = false
  const length = arguments.length

  if (typeof target === 'boolean') {
    deep = target
    target = arguments[1] || {}
    i = 2
  }

  if (Object(target) !== target && type(target) !== 'function') {
    target = {}
  }

  if (length === i) {
    target = this
    i--
  }

  for (; i < length; i++) {
    options = arguments[i]
    if (options !== null) {
      for (name in options) {
        src = target[name]
        copy = options[name]

        if (target === copy) {
          continue
        }
        // eslint-disable-next-line no-cond-assign
        if (deep && copy && (isPlainObject(copy) || (copyIsArray = type(copy) === 'array'))) {
          if (copyIsArray) {
            copyIsArray = false
            clone = src && type(src) === 'array' ? src : []
          } else {
            clone = src && isPlainObject(src) ? src : {}
          }

          target[name] = extend(deep, clone, copy)
        } else if (copy !== undefined) {
          target[name] = copy
        }
      }
    }
  }

  return target
}

const fun = function (baseUrl,app,api) {
  const config = {
    name: 'defaultFetchConfig',
    befores: [
      // 处理地址 默认使用 window.zyConfig.baseUrl
      function (requestInfo) {
        let url = baseUrl ? baseUrl : ''
        requestInfo.url = url + requestInfo.url
        return requestInfo
      },
      // 处理请求头
      function (requestInfo) {
        const refer = {
          headers: {
            'Content-Type': 'application/json',
            token: sessionStorage.getItem('token') || localStorage.getItem('token')
          }
        }
        const mOptions = extend(true, refer, requestInfo.options)
        let body = mOptions.body
        if (body) {
          if (body.toString() === '[object FormData]') {
            delete mOptions.headers['Content-Type']
          } else if (body.toString() === '[object Object]') {
            body = JSON.stringify(body)
          }
          mOptions.body = body
        }
        requestInfo.options = mOptions
        return requestInfo
      }
    ],
    afters: [
      // 区分blob和json
      function (res) {
        if (!res.headers.get('Content-Type')) {
          return res
        }
        const contentType = res.headers.get('Content-Type').toLowerCase()
        if (contentType.includes('application/json')) {
          return res.json()
        } else {
          return res
        }
      },
      // 处理code
      function (res) {
        if (res instanceof Response) {
          const fileNameArr = res.headers.get('content-disposition').replace(/["']/g, '').match(/filename\*?=([^;]+)/)
          const fileName = fileNameArr && fileNameArr.length >= 1
            ? fileNameArr[1]
            : ''
          return {
            fileName: decodeURI(fileName),
            body: res
          }
        } else {
          const jsonData = res
          if (jsonData.code === 200) {
            return jsonData.data
          } else {
            throw new api.CodeError('response code ' + jsonData.code, jsonData)
          }
        }
      }
    ],
    // 取消请求的处理方式
    autoAbort: function (res) {
      if (res.status >= 200 && res.status <= 300) {
        return res
      } else if (res.status === 401) {
        // 处理statuserror会取消请求
        return new api.StatusError('response status ' + res.status, res.status)
      } else {
        // 其他的会照常抛出
        return new Error('response status ' + res.status)
      }
    },
    finalCatch: function (error) {
      if (error instanceof api.StatusError) {
        // 处理 status 的状态 遇到这个问题就跳转登录
        if (app && app.store) {
          app.store.dispatch('user/logout')
        }
        return
      }
      if (error instanceof api.CodeError) {
        // 处理 code 的状态
        throw error
      }
      throw error
    },
    custom: {}
  }
  return config
}

export default fun