/*
 * @Author: qs
 * @Date: 2022-03-26 20:31:58
 * @LastEditTime: 2022-04-26 10:27:02
 * @Description:
 */
import {extend} from 'quasar'
export default {
  name:'temple-config',
  befores:[
    // 处理请求头
    function (requestInfo) {
      const refer = {
        headers: {
          'Content-Type': 'application/json',
          'token':sessionStorage.getItem('token') || localStorage.getItem('token')
        }
      }
      const mOptions = extend(true, refer, requestInfo.options)
      let body = mOptions.body
      if (body) {
        if (body.toString() === '[object FormData]') {
          delete mOptions.headers['Content-Type']
        } else if (body.toString() === '[object Object]') {
          body = JSON.stringify(body)
        }
        mOptions.body = body
      }
      requestInfo.options = mOptions
      return requestInfo
    }
  ],
  afters:[
    // 区分blob和json
    function (res) {
      if (!res.headers.get('Content-Type')) {
        return res
      }
      const contentType = res.headers.get('Content-Type').toLowerCase()
      if (contentType.includes('application/json')) {
        return res.json()
      } else {
        return res
      }
    },
    // 处理code
    function (res) {
      if (res instanceof Response) {
        let fileNameArr = null
        if(res.headers.has('content-disposition')){
          fileNameArr = res.headers.get('content-disposition').replace(/["']/g, '').match(/filename\*?=([^;]+)/)
        }
        const fileName = fileNameArr && fileNameArr.length >= 1
          ? fileNameArr[1]
          : '' + Date.now()
        return res.blob()
              .then(blob=>{
                return {
                  body:blob,
                  fileName: decodeURIComponent(fileName)
                }
              })
      } else {
        const jsonData = res
        if (jsonData.code === 200) {
          return jsonData.data
        } else {
          throw new Error(`response data code ${jsonData.code},返回数据:${jsonData}`)
        }
      }
    }
  ],
    // 取消请求的处理方式
  autoAbort:function (res,{router,store}) {
    if(res.status === 401){
      router && router.push('/login')
      // _mutations
      store && store.hasModule('auth') && store._mutations['auth/logout'] && store.commit('auth/logout')
      return true
    }else{
      return false
    }
  },
  // 添加开发环境提示
  finalCatch:function (err) {
    if(process.env.DEV && window.$z && window.$z.error){
      window.$z.notify.error(`one error not catch (only show when dev ):${err}`)
    }
    return err
  },
  custom:{}
}